/*
** UC: 21111 - Sistemas Operativos
** e-fólio A 2023-24 (fdup.c)
**
** Aluno: 2300163 - Luís Pereira
*/


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/wait.h>
#include <fcntl.h>

int main(int argc, char *argv[]) {
    // Verifica o número de argumentos
    if (argc != 3) {
        fprintf(stderr, "Formato incorrecto. Input: %s <diretoria> <ficheiro>\n", argv[0]);
        exit(EXIT_FAILURE);
    }

    // Abre o diretorio especificado para verificar se o mesmo existe
    FILE *dir_check = fopen(argv[1], "r");
    if (!dir_check) {
        fprintf(stderr, "Diretoria %s não encontrada.\n", argv[1]);
        exit(EXIT_FAILURE);
    }
    fclose(dir_check);

    // Criação dos processos (Comandos e argumentos) em sequência
    char *cmds[] = {"find", "sort", "uniq", "awk"};
    char *args[][9] = {{"find", argv[1], "-type", "f", "-name", argv[2], "-printf", "%h %f\n", NULL},
                       {"sort", "-t", " ", "-k2,2", NULL, NULL, NULL, NULL, NULL},
                       {"uniq", "-f1", "-D", NULL, NULL, NULL, NULL, NULL, NULL},
                       {"awk", "{print $2,$1}", NULL, NULL, NULL, NULL, NULL, NULL, NULL}};
    char proc[] = {'B', 'C', 'D', 'E'};
    char *adress[] = {"/usr/bin/find", "/usr/bin/sort", "/usr/bin/uniq", "/usr/bin/awk"};

    // Cria os arquivos temporários, com os nomes pretendidos
    char *tmp_filenames[] = {"tmp1.txt", "tmp2.txt", "tmp3.txt"};
    int tmp_fds[3];
    for (int i = 0; i < 3; i++) {
        tmp_fds[i] = open(tmp_filenames[i], O_RDWR | O_CREAT | O_TRUNC, 0644);
        if (tmp_fds[i] == -1) {
            perror("Erro ao criar ficheiros temporários");
            exit(EXIT_FAILURE);
        }
    }

    pid_t pid;
    for (int i = 0; i < 4; i++) {
        pid = fork();
        if (pid == -1) {
            perror("Erro ao criar processo");
            exit(EXIT_FAILURE);
        }
        if (pid == 0) {
            // Processo filho
            printf("Processo %c: PID=%d PPID=%d\n", proc[i], getpid(), getppid());
            
            // Executa o comando correspondente ao processo e Redireciona entrada e saída padrão
            if (i == 0) {
                freopen(tmp_filenames[i], "w", stdout);
                execvp(cmds[i], args[i]);
            } else if (i == 1) {
                freopen(tmp_filenames[i-1], "r", stdin); 
                freopen(tmp_filenames[i], "w", stdout); 
                execv(adress[i], args[i]);
            } else if (i == 2) {
		freopen(tmp_filenames[i-1], "r", stdin); 
                freopen(tmp_filenames[i], "w", stdout);
		execlp(cmds[i], args[i][0], args[i][1], args[i][2], NULL);
            } else if (i == 3) {
                freopen(tmp_filenames[i-1], "r", stdin); 
                execl(adress[i], cmds[i], args[i][1], NULL);
            }
            
            // Se chegou aqui, houve um erro ao executar o comando
            perror("Erro ao executar o comando");
            exit(EXIT_FAILURE);
        }
        // Processo pai espera pelo filho
        wait(NULL);
    }

    // Fecha os arquivos temporários
    for (int i = 0; i < 3; i++) {
        close(tmp_fds[i]);
    }

    return 0;
}
